#!/usr/bin/env python

import dotenv
import click
import uuid
import os
import sys
from stdiomask import getpass

from base64 import b64encode as _b64encode, b64decode as _b64decode
import uuid as _uuid

from pathlib import Path

# internals
from . import remote
from . import terminal


_token = {
    "id": _uuid.getnode(),
    "name": "tctl-{uuid}--auto-generated".format(uuid=_uuid.getnode())
}

env_path = Path.home() / '.tradologics'

if not Path.exists(env_path):
    Path.touch(env_path)

dotenv.load_dotenv(env_path)


def delete():
    Path.delete(env_path)
    confirm = input("\nAre you sure you want to delete your credentials? [y/N]: ")
    if confirm not in ["y", "Y"]:
        click.echo("ABORTED")
        sys.exit(0)

    click.echo("\nDeleting... ", nl=False)
    click.echo(click.style("SUCCESS", fg="green"))
    click.echo("NOTE: tctl will need to be re-configured in order to work.")
    sys.exit(0)


def obfuscate(byt, password):
    mask = password.encode()
    lmask = len(mask)
    return bytes(c ^ mask[i % lmask] for i, c in enumerate(byt))


def encrypt(txt):
    return _b64encode(obfuscate(txt.encode(), hex(_token['id']))).decode()


def decrypt(txt):
    return obfuscate(_b64decode(txt.encode()), hex(_token['id'])).decode()


def config(source=None):
    click.echo(terminal.prompt)

    # if message:
    #     click.echo(message)
    #     click.echo("-----------------------------------------------------\n")

    dotenv.load_dotenv(env_path, verbose=True)
    no_token = not os.getenv("TOKEN")

    if no_token:
        click.echo("HELLO 👋")

        if source != "config":
            click.echo("""
tctl isn't configured on this machine yet.

Please have your API Key and Secret Key handy in
order to configure tctl.

Let's get started...

-----------------------------------------------------
""")
        else:
            click.echo("""
Welcome to the tctl!
""")

    else:
        click.echo("""Currently using CustomerId: {customer_id}""".format(
            customer_id=os.getenv("CUSTOMER_ID")))

        update = input("\nReplace it with a new one? [y/N]: ")
        if update not in ["y", "Y"]:
            click.echo("ABORTED")
            sys.exit(0)

    api_key = input("API Key:     ")
    api_secret = getpass("API Secret:  ")

    click.echo("\nValidating... ", nl=False)

    headers = {
        "TGX-API-KEY": api_key,
        "TGX-API-SECRET": api_secret
    }

    tokens = remote.api.get("/tokens?full=true", headers=headers)

    tctl_token = None
    for token in tokens:
        if token["name"] == _token["name"]:
            tctl_token = token
            break

    if not tctl_token:
        res = remote.api.post(
            "/token",
            json={"name": _token['name'], "ttl": -1},
            headers=headers)

        tctl_token = res.get("token")

    # write token
    dotenv.set_key(env_path, "TOKEN", encrypt(tctl_token))

    # get custoemer data
    res = remote.api.get(
        "/me", headers=remote.bearer_token(tctl_token))

    for key in ["name", "email", "customer_id"]:
        dotenv.set_key(env_path, key.upper(), res.get(key))

    click.echo(click.style("SUCCESS\n", fg="green"))
    click.echo("tctl is now configured! 🎉")

    if source == "config":
        click.echo("\nPlease run your command again")



#!/usr/bin/env python

import click
import sys

# internals
from . import version
from . import config
from . import strategies
# from . import tradehooks
# from . import accounts
# from . import aggregated
# from . import marketdata
# from . import tools
# from . import admin


cli = click.CommandCollection(sources=[
    config.cli,
    strategies.cli,
    # tradehooks.cli,
    # accounts.cli,
    # aggregated.cli,
    # marketdata.cli,
    # tools.cli,
    # admin.cli
])


if __name__ == '__main__':
    arg_string = " ".join(sys.argv)
    if "--version" in arg_string:
        click.echo("\ntctl {version}".format(version=version.version))
        click.echo("Copyrights (c) Tradologics, Inc.")
        click.echo("https://tradologics.com/tctl")
        sys.exit()

    cli()



import sys
import os
import requests
from requests.exceptions import ConnectionError
import click

from . import utils
from . import credentials

BASE_URL = "https://api.tradologics.com/beta"


def _parse_response(response_obj):
    # "no content" response
    if "json" not in response_obj.headers.get('Content-Type') and \
            response_obj.status_code in [201, 202, 204]:
        return None

    data = {}
    try:
        data = response_obj.json()
    except Exception:
        click.echo("ERROR: Malformed data returned")
        sys.exit()

    data["status_code"] = response_obj.status_code

    if int(data["status_code"] / 100) != 2:
        error = data.get("errors", [{
            "id": "invalid_request",
            "message": "Cannot process request"
        }])[0]

        click.echo(click.style("FAILED", fg="red"))
        click.echo("\nFailed with status code: {code}".format(
            code=data["status_code"]))

        click.echo(utils.to_json(error))
        sys.exit()

    return data.get("data", {})


def bearer_token():
    token = os.getenv("TOKEN")
    if not token:
        credentials.config()
    token = credentials.decrypt(token)
    return {"Authorization": "Bearer {token}".format(token=token)}


class api:

    @staticmethod
    def get(endpoint, **kwargs):
        url = "{url}{endpoint}".format(url=BASE_URL, endpoint=endpoint)
        if "headers" not in kwargs:
            kwargs["headers"] = bearer_token()
        try:
            r = requests.get(url, **kwargs)
            return _parse_response(r)
        except ConnectionError:
            click.echo("ERROR: Cannot establish connection to Tradologics")
            sys.exit()

    @staticmethod
    def post(endpoint, **kwargs):
        try:
            url = "{url}{endpoint}".format(url=BASE_URL, endpoint=endpoint)
            if "headers" not in kwargs:
                kwargs["headers"] = bearer_token()
            r = requests.post(url, **kwargs)
            return _parse_response(r)
        except ConnectionError:
            click.echo("ERROR: Cannot establish connection to Tradologics")
            sys.exit()

    @staticmethod
    def patch(endpoint, **kwargs):
        try:
            url = "{url}{endpoint}".format(url=BASE_URL, endpoint=endpoint)
            if "headers" not in kwargs:
                kwargs["headers"] = bearer_token()
            r = requests.patch(url, **kwargs)
            return _parse_response(r)
        except ConnectionError:
            click.echo("ERROR: Cannot establish connection to Tradologics")
            sys.exit()

    @staticmethod
    def put(endpoint, **kwargs):
        try:
            url = "{url}{endpoint}".format(url=BASE_URL, endpoint=endpoint)
            if "headers" not in kwargs:
                kwargs["headers"] = bearer_token()
            r = requests.put(url, **kwargs)
            return _parse_response(r)
        except ConnectionError:
            click.echo("ERROR: Cannot establish connection to Tradologics")
            sys.exit()

    @staticmethod
    def delete(endpoint, **kwargs):
        try:
            url = "{url}{endpoint}".format(url=BASE_URL, endpoint=endpoint)
            if "headers" not in kwargs:
                kwargs["headers"] = bearer_token()
            r = requests.delete(url, **kwargs)
            return _parse_response(r)
        except ConnectionError:
            click.echo("ERROR: Cannot establish connection to Tradologics")
            sys.exit()

    @staticmethod
    def options(endpoint, **kwargs):
        try:
            url = "{url}{endpoint}".format(url=BASE_URL, endpoint=endpoint)
            if "headers" not in kwargs:
                kwargs["headers"] = bearer_token()
            r = requests.options(url, **kwargs)
            return _parse_response(r)
        except ConnectionError:
            click.echo("ERROR: Cannot establish connection to Tradologics")
            sys.exit()



#!/usr/bin/env python

import click

# internals
from . import credentials
from . import terminal
from . import utils


@click.group()
def cli():
    pass


@cli.command()
@click.option('--tabulate', '-t', is_flag=True, help="Present results as table.")
def info(tabulate):
    data = [
        {
            "name": "new token",
            "token": "eyJhbGciOiJIU***qNegSkFo",
            "date_expiration": "2050-12-31T00:00:00.000Z",
            "active": True
        },
        {
            "name": "new token i will delete",
            "token": "eyJhbGciOiJIU***QMpgNOxg",
            "date_expiration": "2050-12-31T00:00:00.000Z",
            "active": False
        },
        {
            "name": "tctl-190070690681122--auto-generated",
            "token": "eyJhbGciOiJIU***h9o33UDY",
            "date_expiration": "2050-12-31T00:00:00.000Z",
            "active": True
        },
        {
            "name": "tctl-190070690681122--auto-generated",
            "token": "eyJhbGciOiJIU***ijLMErQQ",
            "date_expiration": "2050-12-31T00:00:00.000Z",
            "active": True
        }]

    if tabulate:
        click.echo(utils.to_table(data))
    else:
        click.echo(utils.to_json(data))


@cli.command()
def logo():
    click.echo(terminal.logo)


@cli.command()
@click.option('--delete', is_flag=True, help="Deletes token from disk.")
def config(delete):
    """Initialize, authorize, and configure the tctl tool.

    Retreives and stores your token in as an encrypted file on disk.
    """
    if delete:
        credentials.delete()

    return credentials.config(source="config")




#!/usr/bin/env python

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="tctl",
    version="0.0.1",
    description="Tradologics Command-line Utility",
    long_description=long_description,
    url="https://github.com/tradologics.com/tctl",
    author="Tradologics, Inc.",
    author_email="opensource@tradologics.com",
    license="Apache 2.0",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 4 - Beta",

        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Office/Business :: Financial",
        "Topic :: Office/Business :: Financial :: Investment",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",

        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    platforms=["any"],
    keywords="tradologics, tradologics.com",
    packages=find_packages(exclude=["contrib", "docs", "tests", "examples"]),
    install_requires=[
        "click>=7.1.2", "requests>=2.24.0", "tabulate>=0.8.3",
        "python-dotenv>=0.14.0", "stdiomask>=0.0.5", "pygments>=2.6.1"
    ],

    py_modules=["tctl"],
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "tctl=tctl:cli",
        ],
    },
    # entry_points="""
    #     [console_scripts]
    #     tctl=tctl:cli
    # """
)




import ujson
from pygments import highlight
from pygments.lexers import JsonLexer
from pygments.formatters import TerminalFormatter
from tabulate import tabulate


def to_table(data, tablefmt="fancy_grid", missingval="?"):
    headers = []
    if isinstance(data, list):
        headers = data[0].keys()
        values = [item.values() for item in data]
    elif isinstance(data, dict):
        headers = data.keys()
        values = data.values()

    headers = [col.replace("_", " ").title() for col in headers]
    return "\n" + tabulate(values, headers=headers,
                        missingval=missingval, tablefmt=tablefmt)


def to_json(obj):
    json_str = ujson.dumps(obj, indent=4, sort_keys=True)
    return "\n" + highlight(json_str, JsonLexer(), TerminalFormatter())


class Struct:
    def __init__(self, **entries):
        self.__dict__.update(entries)
